package jp.co.elnet.mc.action;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Objects;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import jp.co.elnet.mc.dao.HonbunTextDao;
import jp.co.elnet.mc.dto.SystemParamDto;
import jp.co.elnet.mc.entity.TMcArticle;
import jp.co.elnet.mc.entity.TRequestLog;
import jp.co.elnet.mc.exception.McErrorException;
import jp.co.elnet.mc.form.HonbunTextDemandBean;
import jp.co.elnet.mc.form.HonbunTextForm;
import jp.co.elnet.mc.service.DemandService;
import jp.co.elnet.mc.service.McLoggerService;
import jp.co.elnet.mc.service.UserService;
import jp.co.elnet.mc.util.McConstants;
import jp.co.elnet.mc.util.SqlLogGetter;

import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.MessageResourcesUtil;

import com.itextpdf.text.Document;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfCopy;
import com.itextpdf.text.pdf.PdfImportedPage;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * 本文テキスト画面
 *
 * @author yoshimura
 * @since 2015.11.01 yoshimura システム更改(本文テキスト、検索語対応)
 */
public class HonbunTextAction extends BaseAction {

	/** 環境設定情報. */
	@Resource
	private SystemParamDto systemParamDto;

    /** ログ. */
    private  McLoggerService logger = McLoggerService.getLogger(this.getClass().getName());

	/** フォーム */
	@Resource
	@ActionForm
	public HonbunTextForm honbunTextForm;

	/** ユーザー関連処理サービス */
	@Resource
	private UserService userService;

    /** PDF・サムネイル・本文テキスト要求サービスクラス. */
    @Resource
    private DemandService demandService;

    /** サムネイル用Daoクラス. */
    @Resource
    HonbunTextDao honbunTextDao;

    /** レスポンス. */
    @Resource
    protected HttpServletResponse response;

	/**
	 * 本文テキスト表示/印刷処理。
	 * 要求パラメータ．要求区分が"4：本文テキスト表示"の場合は、本文テキスト画面を表示する。
	 * 要求パラメータ．要求区分が"5：本文テキスト印刷"の場合は、本文テキスト印刷要求を行った後、
	 * 本文テキスト準備中画面を表示する。(本文テキスト準備中画面を表示すると、JavaScriptのonLoad関数が実行される。
	 * 関数では、iframe内に"HonbunTextAction.printメソッド"を呼び出し、PDFの読み込みが行われる)
	 *
	 * @return 本文テキスト画面
	 * @throws Exception
	 *
	 */
	@Execute(validator = false)
	public String index() throws Exception
	{
		this.setDataForConfirmCharging();

		if(Objects.nonNull(honbunTextForm.getFromMail())){
			return this.comeFromMail();
		}

		// 原文利用上限値を超えている場合はエラーとする
		if (loginDto.getUseMaxMcCount() <= loginDto.getUseMcCount()) {
			honbunTextForm.setErrorDivUseCountDisp();
			return "error.jsp";
		}

		// 要求パラメータを取得
		String demandDiv = honbunTextForm.getDemandDiv();									// 要求区分(4:表示、5:印刷)
		String elId = honbunTextForm.getDecryptElId(systemParamDto);  						// ELIDの暗号化を解除
		String articleId = honbunTextForm.getArticleId();									// 記事ID
		String processDate = honbunTextForm.getProcessDate();								// 処理日
		String isFromNoSubjectSelectPage = honbunTextForm.getIsFromNoSubjectSelectPage();	// 対象外データ選択画面からの遷移を表すフラグ
		String userId = loginDto.getUserId();												// ユーザーID


		// 対象外データ選択画面からの遷移を表すフラグが渡ってこなかった場合
		if (isFromNoSubjectSelectPage == null) {
			isFromNoSubjectSelectPage = "false";
		}

		// ユーザー種別を取得
		String userType = loginDto.getUserType();
		// ユーザーの表示権限チェック
		if (McConstants.DEMAND_TEXT_DISP.equals(demandDiv) && !userService.canMcTextDisp(userType)) {
			honbunTextForm.setErrorDivAuthDisp();
			return "error.jsp";

		// ユーザーの印刷権限チェック
		} else if (McConstants.DEMAND_TEXT_PRINT.equals(demandDiv) && !userService.canMcTextPrint(userType)) {
			honbunTextForm.setErrorDivAuthPrint();
			return "error.jsp";
		}
		// 本文テキストの印刷権限があるかチェック
		honbunTextForm.setHonbunTextPrintFlg(userService.canMcTextPrint(userType));

		// 書誌データの検索
		TMcArticle tMcArticle = honbunTextDao.selectMcArticle(elId, articleId, processDate, isFromNoSubjectSelectPage);

		// 書誌データが存在し、本文テキストが提供可能であることをチェック
		if (tMcArticle == null || McConstants.NO_TEXT_FLG_NG.equals(tMcArticle.getNoTextFlg())) {
			// 表示要求
			if (McConstants.DEMAND_TEXT_DISP.equals(demandDiv)) {
				honbunTextForm.setErrorDivNoDataDisp();

			// 印刷要求
			} else if (McConstants.DEMAND_TEXT_PRINT.equals(demandDiv)) {
				honbunTextForm.setErrorDivNoDataPrint();
			}
			return "error.jsp";
		}
		honbunTextForm.setMcArticle(tMcArticle);

		// 本文テキストの要求
		HonbunTextDemandBean bean = null;
		if (McConstants.DEMAND_TEXT_DISP.equals(demandDiv)) {
			bean = demandService.mcTextDispDemand(elId, userId, articleId);

		} else if (McConstants.DEMAND_TEXT_PRINT.equals(demandDiv)) {
			bean = demandService.mcTextPrintDemand(elId, userId, articleId);

			// PDFの保護、ページ番号の付与、印刷スクリプトの付与
			createPdf(bean, elId);
		}
		honbunTextForm.setHonbunTextDemandBean(bean);

		// 本文テキスト要求に失敗した場合
		if (!bean.getIsDemandSuccess()) {
			honbunTextForm.setErrorDivCenterDisp();
			return "error.jsp";
		}

		if (McConstants.DEMAND_TEXT_DISP.equals(demandDiv)) {
			// 要求ログの登録
			requestLog(McConstants.REQUEST_DIV_MC_TEXT_DISP, elId, userId, articleId);

			// 本文テキスト画面の表示
			return "index.jsp";
		} else if (McConstants.DEMAND_TEXT_PRINT.equals(demandDiv)) {
			// 要求ログの登録
			requestLog(McConstants.REQUEST_DIV_MC_TEXT_PRINT, elId, userId, articleId);

			// 本文テキスト印刷(本文テキスト準備中画面の表示。画面を表示すると、JavaScriptのonLoad関数が実行される。関数では、iframe内に"HonbunTextAction.printメソッド"を呼び出し、PDFの読み込みが行われる)
			return "print.jsp";
		}

		// ここに来ることはありえない
		return null;
	}

	/**
	 * メールから来た場合に原文利用確認画面に遷移
	 */
	private String comeFromMail(){
		this.honbunTextForm.setFromMail(null);
		return "fromMail.jsp?elId=" + this.honbunTextForm.getElId();
	}
    /**
     *  課金確認メッセージ出力の為のデータ設定処理
     *  20181204 tachikawa 課金確認メッセージの適用範囲
     */
    private void setDataForConfirmCharging() {

    	// システムパラメータを参照
    	honbunTextForm.setOutDemandConfirmMessageDiv( systemParamDto.getOutDemandConfirmMessageDiv()); //メッセージ表示区分
    	honbunTextForm.setOutDemandConfirmMessageCookieLimit(  systemParamDto.getOutDemandConfirmMessageCookieLimit() );//クッキー期限
    	honbunTextForm.setConfirmDemandPrintTextMessage(systemParamDto.getDemandConfirmPrintTextMessage());//印刷用メッセージ
    	honbunTextForm.setConfirmDemandTextMessage(systemParamDto.getDemandConfirmTextMessage());//表示用メッセージ
    	honbunTextForm.setOutDemandConfirmMessageTextCookieLimit(systemParamDto.getOutDemandConfirmMessageTextCookieLimit());//クッキー期限

    }


	/**
	 * セッション内の本文テキストPDFをレスポンスとして返す
	 * (PDF内に印刷用のActionScriptが含まれているため、ブラウザがレスポンスを受け取ると印刷ダイアログが表示される)
	 *
	 * @return 戻り値は常にNULL(レスポンス内に本文テキストPDFを書き込んで返却)
	 * @throws Exception
	 */
	@Execute(validator = false)
	public String print() throws Exception
	{
		String elId = honbunTextForm.getDecryptElId(systemParamDto);  						// ELIDの暗号化を解除
		String userId = loginDto.getUserId();												// ユーザーID

		// PDFファイル名を取得(※フォーマット中に"MILISEC"があったら、現在時刻のミリ秒で置換する)
		String fileNameFormat = systemParamDto.getResponsePdfFileName();
		String fileName = fileNameFormat.replaceAll("MILISEC", String.valueOf(new Date().getTime()));

		OutputStream out = null;
		try {
			HonbunTextDemandBean bean = honbunTextForm.getHonbunTextDemandBean();
			byte[] pdfBytes = bean.getDemandDataPdf();
			response.setContentType("application/pdf");
			response.addHeader("Content-Disposition", "inline; filename=" + fileName);
			response.setHeader("Cache-Control", "cache, must-revalidate");
			response.setHeader("Pragma", "public");
			response.setContentLength(pdfBytes.length);
			out = response.getOutputStream();
			out.write(pdfBytes);
			out.flush();
		} catch (Exception e) {
			String msg = MessageResourcesUtil.getMessage("errors.honbunText.responsePdfError");
			logger.error(msg, e, elId, userId);
			throw e;
		} finally {
			try {
				if (out != null) {
					out.close();
				}
			} catch (Exception e) {
				String msg = MessageResourcesUtil.getMessage("errors.honbunText.responsePdfError");
				logger.error(msg, e, elId, userId);
				throw e;
			}
		}
		return null;
	}

	/**
	 * 要求ログ出力
	 *
	 * @param requestDiv 要求区分
	 * @param elId ELID
	 * @param userId ユーザーID
	 * @param articleId 記事ID
	 * @throws McErrorException
	 */
	private void requestLog(String requestDiv, String elId, String userId, String articleId) throws McErrorException {
		try {
			// 要求ログエンティティクラス
			TRequestLog tRequestLog = new TRequestLog();

			// ELID
			tRequestLog.setElId(elId);

			// ユーザID
			tRequestLog.setUserId(userId);

			// 要求区分
			tRequestLog.setRequestDiv(requestDiv);

			// データ区分
			tRequestLog.setDataDiv(McConstants.DATA_DIV_MC);

			// 記事ID
			tRequestLog.setArticleId(articleId);

			// 要求ログ登録
			honbunTextDao.insertRequestLog(tRequestLog);
		} catch (Exception e) {
			String sql = SqlLogGetter.getLast();
			throw new McErrorException("errors.honbunText.insertRequestLogDbError", new String[] { sql });
		}
	}

    /**
     * 本文テキストのPDF作成処理。
     * 本文テキストのPDF取得APIでは、以下の仕様でPDFが渡ってくる。
     * 　・1つの記事しか要求できない
     * 　・1つの記事が複数ページに渡る場合でも、1つのPDFファイルとして受領
     *
     * 複数ページに渡すことを想定し、PDF右下にページ番号を出力する。
     * PDFには保護をかける。基本的に全て"許可しない"設定とするが、印刷要求のために"印刷許可"とする。
     * また、印刷実行Scriptを付与する。
     *
     * @param kijiBean 記事情報
     * @param elId ELID
     * @throws Exception
     */
    private void createPdf(HonbunTextDemandBean kijiBean, String elId) throws Exception  {
        Document doc = null;
        PdfCopy pdfCopy = null;
        String pdfPageNumDirection = systemParamDto.getPdfPageNumDirection();   // ページ番号の出力方向(横書き or 縦書き)
        int pdfPageNumVertical = systemParamDto.getPdfPageNumVertical();        // ページ番号の縦位置(下からの相対位置)
        int pdfPageNumHorizontal = systemParamDto.getPdfPageNumHorizontal();    // ページ番号の横位置(右からの相対位置)
        String pdfPageNumFont = systemParamDto.getPdfPageNumFont();             // ページ番号のフォント
        int pdfPageNumFontSize = systemParamDto.getPdfPageNumFontSize();        // ページ番号のフォントサイズ
        String pdfPrintScript = systemParamDto.getPdfPrintScript();             // PDF印刷スクリプト
        BaseFont font = BaseFont.createFont(pdfPageNumFont, pdfPageNumDirection, BaseFont.NOT_EMBEDDED);

        try{
            // PDFの結合
            ByteArrayOutputStream joinBaos = new ByteArrayOutputStream();
            // PDFデータにNULLが設定されている場合は処理をスキップ
            if (kijiBean.getDemandDataPdf() == null) {
            	return;
            }

            // PDFのインスタンス生成
            PdfReader pdfReader = new PdfReader(kijiBean.getDemandDataPdf());
            int totalPage = pdfReader.getNumberOfPages();
            if (doc == null) {
                // ドキュメントの作成
                doc = new Document(pdfReader.getPageSizeWithRotation(1));
                pdfCopy = new PdfCopy(doc, joinBaos);
                doc.open();
            }

            // １ページづつ追加
            for (int pageIdx = 1; pageIdx <= totalPage; pageIdx++) {
                PdfImportedPage page = pdfCopy.getImportedPage(pdfReader, pageIdx);
                pdfCopy.addPage(page);
            }
            doc.close();

            // ページ番号の出力準備
            ByteArrayOutputStream pageBaos = new ByteArrayOutputStream();
            pdfReader = new PdfReader(joinBaos.toByteArray());
            totalPage = pdfReader.getNumberOfPages();
            PdfStamper stamp = new PdfStamper(pdfReader, pageBaos);

            // PDFの保護を設定。また、印刷実行スクリプトも設定。
            stamp.setEncryption(null, null, PdfWriter.ALLOW_PRINTING, PdfWriter.STANDARD_ENCRYPTION_128);
            stamp.addJavaScript(pdfPrintScript);

            // ページ番号の出力
            int pageNum = 0;
            for (int pageIdx = 1; pageIdx <= totalPage; pageIdx++) {
                pageNum++;
                PdfContentByte over = stamp.getOverContent(pageIdx);
                over.saveState();
                over.beginText();
                over.setFontAndSize(font, pdfPageNumFontSize);
                Rectangle r = pdfReader.getPageSize(pageIdx);
                over.showTextAligned(PdfContentByte.ALIGN_LEFT, String.valueOf(pageNum),
                                     r.getWidth() - pdfPageNumHorizontal, pdfPageNumVertical, 0); // 右下
                over.endText();
                over.restoreState();
            }
            stamp.close();
            kijiBean.setDemandDataPdf(pageBaos.toByteArray());
        } catch (Exception e) {
            String userId = loginDto.getUserId();
            String msg = MessageResourcesUtil.getMessage("errors.honbunText.createPdfError");
            logger.error(msg, e, elId, userId);
            throw e;
        }
    }
}
