package jp.co.elnet.mc.action;

import java.io.ByteArrayOutputStream;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.servlet.http.HttpServletRequest;
import javax.transaction.UserTransaction;

import jp.co.elnet.mc.dao.PdfDemandDao;
import jp.co.elnet.mc.dto.SystemParamDto;
import jp.co.elnet.mc.entity.TKpArticle;
import jp.co.elnet.mc.entity.TMcArticle;
import jp.co.elnet.mc.entity.TRequestLog;
import jp.co.elnet.mc.form.KijiBean;
import jp.co.elnet.mc.form.MlLoginForm;
import jp.co.elnet.mc.form.PdfDemandBean;
import jp.co.elnet.mc.form.WaitingDemandForm;
import jp.co.elnet.mc.service.DemandService;
import jp.co.elnet.mc.service.KijiListService;
import jp.co.elnet.mc.service.McLoggerService;
import jp.co.elnet.mc.service.RequestLogThreadService;
import jp.co.elnet.mc.service.UserService;
import jp.co.elnet.mc.util.McConstants;
import jp.co.elnet.mc.util.SqlLogGetter;

import org.seasar.framework.aop.annotation.RemoveSession;
import org.seasar.framework.beans.util.Beans;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.MessageResourcesUtil;

import com.itextpdf.text.Document;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfCopy;
import com.itextpdf.text.pdf.PdfImportedPage;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * PDF表示/印刷要求確認画面
 * @author yoshimura
 *
 */
public class WaitingDemandAction extends BaseAction {

    /** 環境設定情報. */
    @Resource
    private SystemParamDto systemParamDto;

    /** ログ. */
    private  McLoggerService logger = McLoggerService.getLogger(this.getClass().getName());

    /** メールファイル用ログイン画面Form(※セッションスコープとして前画面で生成される). */
    @Resource
    public MlLoginForm mlLoginForm;

    /** PDF表示/印刷要求確認画面Form. */
    @Resource
    @ActionForm
    public WaitingDemandForm waitingDemandForm;

    /** トランザクション. */
    @Resource
    protected UserTransaction tran;

    /** DB操作. */
    @Resource
    private PdfDemandDao pdfDemandDao;

    /** ユーザー関連処理サービス */
    @Resource
    private UserService userService;

    /** 記事一覧画面サービスクラス. */
    @Resource
    private KijiListService kijiListService;

    /** PDF・サムネイル要求サービスクラス. */
    @Resource
    private DemandService demandService;

    /** リクエスト. */
    @Resource
    protected HttpServletRequest request;

    /** エラーメッセージ ※JSP内で使用する */
    public List<String> errorMessages;

    /** PDF作成成功件数 ※JSP内で使用する */
    public int successCnt = 0;

    /** PDF作成失敗件数 ※JSP内で使用する */
    public int errorCnt = 0;

    /** 画面表示用の記事情報リスト ※JSP内で使用する */
    public List<PdfDemandBean> kijiBeanList;

    /** 画面表示用の要求区分によって画面の文言を切り替えるためのテキスト ※JSP内で使用する */
    public String modeText;

    /** 要求日時のフォーマット(複数部印刷対応) */
    private static final String DATE_FORMAT = "yyyy/MM/dd HH:mm:ss.SSS";

    /**
     * 初期画面表示。
     * この画面には、以下のパターンで遷移してくる。
     * 　・メールファイルから表示/印刷要求され、既にログインしていたため、メール用ログイン画面からリダイレクトした場合
     * 　・メールファイルから表示/印刷要求され、まだログインしていなくて、メール用ログイン画面からログインして遷移した場合
     * 　・記事一覧画面などから表示/印刷要求された場合
     *
     * メールファイルから遷移してきた場合は、メール用フォームがセッションスコープで保持される。
     */
    @Execute(validator = false, input="/waitingDemand/index.jsp")
    @RemoveSession(name = "mlLoginForm")
    @TransactionAttribute(TransactionAttributeType.NEVER)
    public String index() throws Exception {
        // メール用フォーム(セッション)から要求区分の取得を試みる。取得できない場合は、PDF確認フォームから遷移してきたものとする
        String demandDiv = mlLoginForm.getDemandDiv();
        if (demandDiv == null) {
            // 記事一覧画面から渡された要求区分を取得(※記事IDなどは既にフォーム内に格納されている)
            demandDiv = waitingDemandForm.getDemandDiv();
        } else {
            // ELIDはセッションスコープに含まれていないため、クエリストリングから取得したELIDを設定する
            mlLoginForm.setElId(waitingDemandForm.getElId());
            // メールファイルからのパラメータをコピー
            Beans.copy(mlLoginForm, waitingDemandForm).execute();
        }
        // カンマ区切りで渡ってきた記事データを項目毎に展開。また、記事データは不要になるのでNULLを設定。(※必ずNULLを設定すること。なぜなら、記事一覧画面から確認画面を表示後、記事一覧画面に戻ってサムネイルを表示し、サムネイルから原文表示(又は印刷)を実行した場合、記事データが残っていると、そちらを表示してしまう不具合が発生したため、必ずNULLを設定する必要がある)
        waitingDemandForm.splitArticleData();
        waitingDemandForm.setArticleData(null);

        // 要求区分がサムネイルの場合(※メールファイルから遷移してきた場合のみ)
        if (McConstants.DEMAND_THUMBNAIL.equals(demandDiv)) {
            return "/thumbnail/indexMail?elId=" + waitingDemandForm.getElId()
                    + "&processDate=" + waitingDemandForm.getProcessDate()[0]
                    + "&dataDiv=" + waitingDemandForm.getDataDiv()[0]
                    + "&articleId=" + waitingDemandForm.getArticleId()[0];
        }



        // 記事リスト生成(※書誌データ取得、及び、権限や原文提供不可などのチェック)
        kijiBeanList = createKijiBeanList(demandDiv);


		// 原文利用上限値を超えている場合はエラーとする
		boolean isMcChecked = false;
		boolean isKpChecked = false;
		for (int i = 0; i < kijiBeanList.size(); i++) {
			PdfDemandBean kijiBean = kijiBeanList.get(i);
			String currentDataDiv = kijiBean.getDataDiv();
			if (!isMcChecked && McConstants.DATA_DIV_MC.equals(currentDataDiv)) {
				isMcChecked = true;
				if (loginDto.getUseMaxMcCount() <= loginDto.getUseMcCount()) {
					return "useCountError.jsp";
				}
			}
			if (!isKpChecked && McConstants.DATA_DIV_KP.equals(currentDataDiv)) {
				isKpChecked = true;
				if (loginDto.getUseMaxKpCount() <= loginDto.getUseKpCount()) {
					return "useCountError.jsp";
				}
			}
			// MC、KPともにチェック済の場合は、以降の処理をスキップ
			if (isMcChecked && isKpChecked) {
				break;
			}
		}

        // 記事リストの中で、MC記事の要求が連続している場合に、1つの要求リストとした集約リストを取得する
        List<List<PdfDemandBean>> aggregateList = getDemandAggregateList(kijiBeanList);

        // PDF要求実行
        String decElId = waitingDemandForm.getDecryptElId(systemParamDto);  // ELIDの暗号化を解除
        String userId = loginDto.getUserId();
        pdfDemand(decElId, userId, aggregateList);

    	// KP分の要求ログ登録。MC分については、execute()内で登録する。
    	insertKpRequestLog(decElId, userId, demandDiv, kijiBeanList);
        // PDF作成
        createPdf(kijiBeanList, demandDiv);



        // 画面表示用の文言を設定
        if (McConstants.DEMAND_PDF.equals(demandDiv)) {
            modeText = "表示";
        } else {
            modeText = "印刷";
        }

        // ユーザーに表示 or 印刷権限がない場合、エラーメッセージを取得。権限がある場合は空のArrayListが返却される
        errorMessages = getAuthErrorMessage(kijiBeanList);

        // PDF要求の成功・失敗件数をカウント(※この件数を元に、JSP側で要求ボタンの表示制御などを行う)
        setResultCount(kijiBeanList);

        // 確認画面Form(※セッションスコープ)に要求結果をコピーする
        // (※コピーした要求結果は、execute()内で、要求が成功したMC記事の要求ログを登録する際に使用する)
        waitingDemandForm.createResultCopy(kijiBeanList);

        // エラーが0件の場合はPDFを表示。1件以上の場合は確認画面を表示。
        if (errorCnt == 0) {
            //return getDispPdfUrl();   // del yoshimura 2011.11.04 確認画面を表示する必要が無い場合でも、必ず確認画面を表示し、確認画面JSP内で全件成功していたら別ウィンドウを表示する仕様になった
            return "index.jsp";
        }
        return "index.jsp";
    }


    /**
     * 表示/印刷実行
     */
    @Execute(validator = false, redirect = true)
    @TransactionAttribute(TransactionAttributeType.NEVER)
    public String execute() throws Exception {
        String decElId = waitingDemandForm.getDecryptElId(systemParamDto);  // ELIDの暗号化を解除
        String userId = loginDto.getUserId();

        // 要求日時を取得
    	waitingDemandForm.setRequestDatetime(getDate().toString());

        // 確認画面Form(セッション)から要求区分、記事ID、データ区分、要求結果を取得
        String demandDiv = waitingDemandForm.getDemandDiv();
        String[] articleIds = waitingDemandForm.getArticleId();
        String[] dataDivs = waitingDemandForm.getDataDiv();
        String[] createResult = waitingDemandForm.getCreateResult();

        // 複数部印刷のデータを取得(複数部印刷対応)
        // 部数入力表示フラグ
        String inpPrnNumFlg = waitingDemandForm.getInpPrnNumFlg();
        // 印刷部数
        String printNum = waitingDemandForm.getPrintNum();

        // 複数部印刷画面からの遷移チェック
        if(inpPrnNumFlg != null && printNum != null) {

        	// 複数部印刷画面からの遷移の場合

        	// 【注意】
        	// 本処理では、登録データの作成のみでDBへの登録は行わない
        	// DBへの登録処理は別スレッドを生成し、そのスレッド内で登録処理を行う。

        	// 登録用のリスト生成
        	List<TRequestLog>	insertList = new ArrayList<TRequestLog>();

        	// 日時を取得
        	Timestamp ts = getDate();
        	// 部数をint型に変換
        	int nPrintnum = Integer.parseInt(printNum);

        	// MC記事、かつ、要求が成功しているものだけ要求ログの登録データを作成する
        	for (int i = 0; i < articleIds.length; i++) {
        		if (McConstants.DATA_DIV_MC.equals(dataDivs[i]) &&
        			McConstants.DEMAND_SUCCESS.equals(createResult[i])) {
        			// 部数分ループ処理
        			for (int num = 0; num < nPrintnum; num++){
        				// 登録データを作成し、リストに追加する
        				insertList.add(createRequestLogData(decElId, userId, articleIds[i], demandDiv, dataDivs[i], ts));
        			}
        		}
        	}

        	// 部数入力表示フラグと印刷部数をnullで初期化する
        	waitingDemandForm.setInpPrnNumFlg(null);
        	waitingDemandForm.setPrintNum(null);

        	// スレッド開始
        	new RequestLogThreadService(decElId, userId, ts, insertList, tran, pdfDemandDao).start();

        }else {

        	// 複数部印刷画面以外からの遷移の場合

        	try{
        		tran.begin();
        		// MC記事、かつ、要求が成功しているものだけ要求ログを登録する
        		// ※日経分については、PDF要求時に既に登録しているので、ここでは行わない
        		for (int i = 0; i < articleIds.length; i++) {
        			if (McConstants.DATA_DIV_MC.equals(dataDivs[i]) &&
        				McConstants.DEMAND_SUCCESS.equals(createResult[i])) {
        				insertRequestLog(decElId, userId, articleIds[i], demandDiv, dataDivs[i], null);
        			}
        		}
        		tran.commit();
        	} catch (Exception e) {
        		String sql = SqlLogGetter.getLast();
        		String msg = MessageResourcesUtil.getMessage("errors.demandConfirm.insertRequestLogDbError", sql);
        		logger.error(msg, e, decElId, userId);
        		tran.rollback();
        		throw e;
        	}
        }

        // PDFを表示
        return getDispPdfUrl();
    }

    /**
     * PDF表示/印刷画面へのURLを取得する
     *
     * @return PDF表示/印刷画面へのURL
     */
    private String getDispPdfUrl() {
        return "/dispPdf/?elId=" + waitingDemandForm.getElId();
    }

    /**
     * 記事リストを生成する。
     * 記事リストには以下のデータを設定する。
     * 　・書誌データ
     * 　・要求区分に応じた処理が実行可能かのチェック結果
     * 　　(ユーザーに権限が無い場合 or 書誌データが無い場合 or 削除フラグが有効 or 原文提供不可などをチェック)
     *
     * @param demandDiv 要求区分
     * @return 記事リスト
     */
    private List<PdfDemandBean> createKijiBeanList(String demandDiv){
        // リクエストから受け取った記事情報をBeanに変換。また、ユーザーの権限チェックを実施
        kijiBeanList = new ArrayList<PdfDemandBean>();
        String decElId = waitingDemandForm.getDecryptElId(systemParamDto);  // ELIDの暗号化を解除
        String[] processDates = waitingDemandForm.getProcessDate(); // 記事毎の処理日
        String[] dataDivs = waitingDemandForm.getDataDiv();         // 記事毎のデータ区分
        String[] articleIds = waitingDemandForm.getArticleId();     // 記事毎の記事ID
        for (int i = 0; i < articleIds.length; i++) {
            PdfDemandBean kijiBean = new PdfDemandBean();
            kijiBean.setProcessDate(processDates[i]);               // 処理日
            kijiBean.setDataDiv(dataDivs[i]);                       // データ区分
            kijiBean.setArticleId(articleIds[i]);                   // 記事ID
            kijiBean.setResult(McConstants.DEMAND_SUCCESS);         // PDF作成結果を"成功"状態で初期化

            // MC記事の場合
            if (McConstants.DATA_DIV_MC.equals(dataDivs[i])) {
                // 書誌を取得
                TMcArticle article = pdfDemandDao.selectMcArticle(decElId, processDates[i], articleIds[i]);
                if (article == null) {
                    kijiBean.setResult(McConstants.DEMAND_ERROR_NO_DATA);
                    kijiBeanList.add(kijiBean);
                    continue;
                }

                // 確認画面表示用に、書誌データをBeanに設定
                KijiBean tmpBean = kijiListService.commonCreateMcKiji(article);
                Beans.copy(tmpBean, kijiBean).execute();
                kijiBeanList.add(kijiBean);

                // ユーザー権限が無い or 削除フラグが有効 or 原文提供不可の場合
                if (!canMcDemandCheck(demandDiv, article, kijiBean)) {
                    continue;
                }
            }

            // KP記事の場合
            if (McConstants.DATA_DIV_KP.equals(dataDivs[i])) {
                // 書誌を取得
                TKpArticle article = pdfDemandDao.selectKpArticle(decElId, processDates[i], articleIds[i]);
                if (article == null) {
                    kijiBean.setResult(McConstants.DEMAND_ERROR_NO_DATA);
                    kijiBeanList.add(kijiBean);
                    continue;
                }

                // 確認画面表示用に、書誌データをBeanに設定
                KijiBean tmpBean = kijiListService.commonCreateKpKiji(article);
                Beans.copy(tmpBean, kijiBean).execute();
                kijiBean.setImageUrl(article.getImageUrl());
                kijiBeanList.add(kijiBean);

                // ユーザー権限が無い or 削除フラグが有効 or 原文URLが無いの場合
                if (!canKpDemandCheck(demandDiv, article, kijiBean)) {
                    continue;
                }
            }
        }
        return kijiBeanList;
    }

    /**
     * 指定されたMC記事のPDF表示（又は印刷）が可能かチェックする。
     * ユーザー権限が無い or 削除フラグが有効 or 原文提供不可の場合に要求不可とみなす。
     * @param demandDiv 要求区分
     * @param article 記事データ
     * @param kijiBean 記事リスト
     * @return 指定されたMC記事のPDF表示（又は印刷）が可能な場合にtrue
     */
    private boolean canMcDemandCheck(String demandDiv, TMcArticle article, PdfDemandBean kijiBean){
        // ユーザーの権限チェック
        String userType = loginDto.getUserType();           // ユーザー種別を取得
        if (McConstants.DEMAND_PDF.equals(demandDiv)
                && !userService.canMcPdfDisp(userType)) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_MC_AUTH_PDF);
            return false;
        }
        if (McConstants.DEMAND_PRINT.equals(demandDiv)
                && !userService.canMcPdfPrint(userType)) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_MC_AUTH_PRINT);
            return false;
        }

        // 削除フラグが"有効"以外 or 原文提供不可の場合
        boolean IsFromNoSubjectSelectPage = Boolean.valueOf(waitingDemandForm.getIsFromNoSubjectSelectPage()); //2014年機能追加対応 対象外データ選択画面からの処理化の判定を追加 2014.10.09 inoue
        if (!McConstants.ARTICLE_DELFLG_ENABLE.equals(article.getDelFlg())	&& !IsFromNoSubjectSelectPage)
        {
            kijiBean.setResult(McConstants.DEMAND_ERROR_DELETE);
            return false;

        } else if (McConstants.NO_IMAGE_FLG_NG.equals(article.getNoImageFlg())) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_NO_IMAGE);
            return false;
        }



        return true;
    }

    /**
     * 指定された日経記事のPDF表示（又は印刷）が可能かチェックする。
     * ユーザー権限が無い or 削除フラグが有効 or 原文URLが無い場合に要求不可とみなす。
     * @param demandDiv 要求区分
     * @param article 記事データ
     * @param kijiBean 記事リスト
     * @return 指定されたKP記事のPDF表示（又は印刷）が可能な場合にtrue
     */
    private boolean canKpDemandCheck(String demandDiv, TKpArticle article, PdfDemandBean kijiBean){
        // ユーザーの権限チェック
        String kpAuth = loginDto.getKpAuth();               // KP利用権限を取得
        if (McConstants.DEMAND_PDF.equals(demandDiv)
                && !userService.canKpPdfDisp(kpAuth)) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_KP_AUTH_PDF);
            return false;
        }
        if (McConstants.DEMAND_PRINT.equals(demandDiv)
                && !userService.canKpPdfPrint(kpAuth)) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_KP_AUTH_PRINT);
            return false;
        }

        // 削除フラグが"有効"以外 or 原文PDFへのURLが無い場合
        boolean IsFromNoSubjectSelectPage = Boolean.valueOf(waitingDemandForm.getIsFromNoSubjectSelectPage()); //2014年機能追加対応 対象外データ選択画面からの処理化の判定を追加 2014.10.09 inoue
        if (!McConstants.ARTICLE_DELFLG_ENABLE.equals(article.getDelFlg()) && !IsFromNoSubjectSelectPage) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_DELETE);
            return false;
        } else if (StringUtil.isEmpty(article.getImageUrl())) {
            kijiBean.setResult(McConstants.DEMAND_ERROR_NO_IMAGE);
            return false;
        }
        return true;
    }

    /**
     * 引数の記事リストの中で、MC記事の要求が連続している場合に
     * 1つの要求リストとした集約リストを取得する。
     * MC記事が連続していない場合や、日経記事の場合は単独の要求として集約リストに追加する。
     *
     * @param kijiBeanList 記事リスト
     * @return 集約リスト
     */
    private List<List<PdfDemandBean>> getDemandAggregateList(List<PdfDemandBean> kijiBeanList) {
        List<List<PdfDemandBean>> aggregateList = new ArrayList<List<PdfDemandBean>>();
        List<PdfDemandBean> tmpList = new ArrayList<PdfDemandBean>();
        String prevDataDiv = null;

        for (int i = 0; i < kijiBeanList.size(); i++) {
            PdfDemandBean kijiBean = kijiBeanList.get(i);
            String currentDataDiv = kijiBean.getDataDiv();

            // 日経記事 or 直前の記事とデータ区分が異なる場合はaggregateListに追加
            if (McConstants.DATA_DIV_KP.equals(currentDataDiv) ||
                !kijiBean.getDataDiv().equals(prevDataDiv)) {
                if (i != 0) {
                    aggregateList.add(tmpList);
                }
                tmpList = new ArrayList<PdfDemandBean>();
            }
            tmpList.add(kijiBean);
            prevDataDiv = currentDataDiv;

            // 最終データの場合はaggregateListに追加
            if (i == kijiBeanList.size() - 1) {
                aggregateList.add(tmpList);
            }
        }
        return aggregateList;
    }

    /**
     * PDF要求を実行する。
     * 要求結果は引数aggregateListに設定する
     *
     * @param elId ELID
     * @param userId ユーザーID
     * @param aggregateList MC要求を集約した記事リスト
     */
    private void pdfDemand(String elId , String userId ,
            List<List<PdfDemandBean>> aggregateList) throws Exception {

        for (List<PdfDemandBean> kijiBeanList : aggregateList) {
            PdfDemandBean kijiBean = kijiBeanList.get(0);
            String dataDiv = kijiBean.getDataDiv();
            if (McConstants.DATA_DIV_MC.equals(dataDiv)) {
                demandService.mcPdfDemand(elId, userId, kijiBeanList);
            } else {
                demandService.kpPdfDemand(elId, userId, kijiBean);
            }
        }
    }

    /**
     * KP分の要求ログを登録する。
     * MC分については、execute()内で登録するので、ここでは登録しない。
     *
     * @param elId ELID
     * @param userId ユーザーID
     * @param demandDiv 要求区分(PDF表示 or 印刷)
     * @param kijiBeanList 記事リスト
     */
    private void insertKpRequestLog(String elId , String userId ,
            String demandDiv , List<PdfDemandBean> kijiBeanList) throws Exception {

// 2011.11.21 del yoshimura 全ての要求が成功したか否かに関わらず、MC分の登録はexecute()内で行うようにした
//        // 全ての要求が成功したか判定
//        boolean allDemandSucess = true;
//        for (PdfDemandBean kijiBean : kijiBeanList) {
//            if (!kijiBean.getIsDemandSuccess()) {
//                allDemandSucess = false;
//                break;
//            }
//        }

        try {
            tran.begin();

            for (PdfDemandBean kijiBean : kijiBeanList) {
                String articleId = kijiBean.getArticleId(); // 記事ID
                String dataDiv = kijiBean.getDataDiv();     // データ区分(MC or KP)
                String result = kijiBean.getResult();       // 要求結果

// 2011.11.21 del yoshimura 全ての要求が成功したか否かに関わらず、MC分の登録はexecute()内で行うようにした
//                // 全ての要求が成功した場合は直接PDFが表示されるので、MC分の要求ログを登録する
//                if (allDemandSucess &&
//                    McConstants.DATA_DIV_MC.equals(dataDiv)) {
//
//                    insertRequestLog(elId, userId, articleId, demandDiv, dataDiv);
//                }

                // 日経分の要求ログを登録する(※要求に失敗しても登録する。ただし、日経に要求していない場合(権限エラーなど)は登録しない)
                if (McConstants.DATA_DIV_KP.equals(kijiBean.getDataDiv()) &&
                    (McConstants.DEMAND_SUCCESS.equals(result) ||
                     McConstants.DEMAND_ERROR_REQUEST.equals(result) ||
                     McConstants.DEMAND_ERROR_PDF_CREATE.equals(result))) {

                    // 複数部印刷対応
                    insertRequestLog(elId, userId, articleId, demandDiv, dataDiv, null);
                }
            }

            tran.commit();
        } catch (Exception e) {
        	String sql = SqlLogGetter.getLast();
            String msg = MessageResourcesUtil.getMessage("errors.demandConfirm.insertRequestLogDbError", sql);
            logger.error(msg, e, elId, userId);
            tran.rollback();
            throw e;
        }
    }


    /**
     * 要求ログテーブルに使用するデータを作成する
     *
     * @param elId ELID
     * @param userId ユーザーID
     * @param articleId 記事ID
     * @param demandDiv 要求区分(PDF表示 or 印刷)
     * @param dataDiv データ区分(MC or KP)
     * @param requestDatetime 要求日時
     */
    private TRequestLog createRequestLogData(String elId , String userId ,
    		String articleId , String demandDiv , String dataDiv, Timestamp requestDatetime) {

    	// 戻り値用
    	TRequestLog ret = new TRequestLog();

    	// ELID
    	ret.setElId(elId);
    	// ユーザID
    	ret.setUserId(userId);
    	// 記事ID
    	ret.setArticleId(articleId);
    	// 要求日時
    	ret.setRequestDatetime(requestDatetime);
    	// 要求区分
    	if (McConstants.DATA_DIV_MC.equals(dataDiv)){
    		if (McConstants.DEMAND_PDF.equals(demandDiv)) {
    			ret.setRequestDiv(McConstants.REQUEST_DIV_MC_PDF_DISP);
    		} else {
    			ret.setRequestDiv(McConstants.REQUEST_DIV_MC_PDF_PRINT);
    		}
    		ret.setDataDiv(McConstants.DATA_DIV_MC);
    	} else {
    		if (McConstants.DEMAND_PDF.equals(demandDiv)) {
    			ret.setRequestDiv(McConstants.REQUEST_DIV_KP_PDF_DISP);
    		} else {
    			ret.setRequestDiv(McConstants.REQUEST_DIV_KP_PDF_PRINT);
    		}
    		ret.setDataDiv(McConstants.DATA_DIV_KP);
    	}

    	return ret;
    }

    /**
     * 要求ログを登録する
     *
     * @param elId ELID
     * @param userId ユーザーID
     * @param articleId 記事ID
     * @param demandDiv 要求区分(PDF表示 or 印刷)
     * @param dataDiv データ区分(MC or KP)
     * @param requestDatetime 要求日時
     * @throws Exception
     */
    private void insertRequestLog(String elId , String userId , String articleId , String demandDiv , String dataDiv, Timestamp requestDatetime) throws Exception {
    	// 登録データを作成
    	TRequestLog requestLog = createRequestLogData(elId, userId, articleId, demandDiv, dataDiv,requestDatetime);

		// 登録処理
		pdfDemandDao.insertRequestLog(requestLog);
    }

    /**
     * PDF作成処理。
     * 各記事のPDFを結合した後、PDF右下にページ番号を出力する。
     * PDFには保護をかける。基本的に全て"許可しない"設定とするが、印刷要求時には"印刷許可"とする。
     * 印刷要求の場合は、印刷実行Scriptを付与する。
     *
     * @param kijiBeanList 記事リスト
     * @param demandDiv 要求区分(PDF表示 or 印刷)
     */
    private void createPdf(List<PdfDemandBean> kijiBeanList, String demandDiv) throws Exception  {
        Document doc = null;
        PdfCopy pdfCopy = null;
        String pdfPageNumDirection = systemParamDto.getPdfPageNumDirection();   // ページ番号の出力方向(横書き or 縦書き)
        int pdfPageNumVertical = systemParamDto.getPdfPageNumVertical();        // ページ番号の縦位置(下からの相対位置)
        int pdfPageNumHorizontal = systemParamDto.getPdfPageNumHorizontal();    // ページ番号の横位置(右からの相対位置)
        String pdfPageNumFont = systemParamDto.getPdfPageNumFont();             // ページ番号のフォント
        int pdfPageNumFontSize = systemParamDto.getPdfPageNumFontSize();        // ページ番号のフォントサイズ
        String pdfPrintScript = systemParamDto.getPdfPrintScript();             // PDF印刷スクリプト
        BaseFont font = BaseFont.createFont(pdfPageNumFont, pdfPageNumDirection, BaseFont.NOT_EMBEDDED);

        try{
            // PDFの結合
            ByteArrayOutputStream joinBaos = new ByteArrayOutputStream();
            for (PdfDemandBean kijiBean : kijiBeanList) {
                // PDFデータにNULLが設定されている場合(※1)は処理をスキップ
                // ※1　MC記事をまとめて要求した場合、センターからは1つのPDFデータとして結合された状態で送られてくる。
                // 　　このPDFデータは、まとめて要求した時の先頭にあったkijiBeanListにのみ設定される
                if (kijiBean.getDemandData() == null) {
                    continue;
                }

                // PDFのインスタンス生成
                PdfReader pdfReader = new PdfReader(kijiBean.getDemandData());
                int totalPage = pdfReader.getNumberOfPages();
                if (doc == null) {
                    // ドキュメントの作成
                    doc = new Document(pdfReader.getPageSizeWithRotation(1));
                    pdfCopy = new PdfCopy(doc, joinBaos);
                    doc.open();
                }

                // １ページづつ追加
                for (int pageIdx = 1; pageIdx <= totalPage; pageIdx++) {
                    PdfImportedPage page = pdfCopy.getImportedPage(pdfReader, pageIdx);
                    pdfCopy.addPage(page);
                }
            }
            if (doc == null) {
                // PDF要求に成功している記事が無い場合は何もしない
                return;
            }
            doc.close();



            // ページ番号の出力準備
            ByteArrayOutputStream pageBaos = new ByteArrayOutputStream();
            PdfReader pdfReader = new PdfReader(joinBaos.toByteArray());
            int totalPage = pdfReader.getNumberOfPages();
            PdfStamper stamp = new PdfStamper(pdfReader, pageBaos);

            // PDFの保護を設定。印刷要求の場合は、印刷実行スクリプトも設定。
            if (McConstants.DEMAND_PRINT.equals(demandDiv)) {
                stamp.setEncryption(null, null, PdfWriter.ALLOW_PRINTING, PdfWriter.STANDARD_ENCRYPTION_128);
                stamp.addJavaScript(pdfPrintScript);
            } else {
                stamp.setEncryption(null, null, 0, PdfWriter.STANDARD_ENCRYPTION_128);
            }

            // ページ番号の出力
            int pageNum = 0;
            for (int pageIdx = 1; pageIdx <= totalPage; pageIdx++) {
                pageNum++;
                PdfContentByte over = stamp.getOverContent(pageIdx);
                over.saveState();
                over.beginText();
                over.setFontAndSize(font, pdfPageNumFontSize);
                Rectangle r = pdfReader.getPageSize(pageIdx);
                over.showTextAligned(PdfContentByte.ALIGN_LEFT, String.valueOf(pageNum),
                                     r.getWidth() - pdfPageNumHorizontal, pdfPageNumVertical, 0); // 右下
                over.endText();
                over.restoreState();
            }
            stamp.close();
            waitingDemandForm.setPdfBytes(pageBaos.toByteArray());
        } catch (Exception e) {
            String decElId = waitingDemandForm.getDecryptElId(systemParamDto);  // ELIDの暗号化を解除
            String userId = loginDto.getUserId();
            String msg = MessageResourcesUtil.getMessage("errors.demandConfirm.createPdfError");
            logger.error(msg, e, decElId, userId);
            throw e;
        }
    }

    /**
     * ユーザーに表示 or 印刷権限がない場合、エラーメッセージを取得。
     * エラーが無い場合は、空のメッセージリストを返す。
     *
     * @param kijiBeanList 記事リスト
     * @return 権限エラーメッセージ
     */
    private ArrayList<String> getAuthErrorMessage(List<PdfDemandBean> kijiBeanList) {
        // エラー種別と対応するエラーメッセージを定義
        List<String> errorTypeList = new ArrayList<String>();
        errorTypeList.add(McConstants.DEMAND_ERROR_MC_AUTH_PDF);    // 要求失敗 - MC記事のPDF表示権限なし
        errorTypeList.add(McConstants.DEMAND_ERROR_MC_AUTH_PRINT);  // 要求失敗 - MC記事のPDF印刷権限なし
        errorTypeList.add(McConstants.DEMAND_ERROR_KP_AUTH_PDF);    // 要求失敗 - 日経記事のPDF表示権限なし
        errorTypeList.add(McConstants.DEMAND_ERROR_KP_AUTH_PRINT);  // 要求失敗 - 日経記事のPD印刷示権限なし

        List<String> errorMessageList = new ArrayList<String>();
        errorMessageList.add(MessageResourcesUtil.getMessage("errors.demandConfirm.noAuthorityMcPdf"));
        errorMessageList.add(MessageResourcesUtil.getMessage("errors.demandConfirm.noAuthorityMcPrint"));
        errorMessageList.add(MessageResourcesUtil.getMessage("errors.demandConfirm.noAuthorityKpPdf"));
        errorMessageList.add(MessageResourcesUtil.getMessage("errors.demandConfirm.noAuthorityKpPrint"));


        // 記事リスト内に権限エラーが含まれていた場合、対応するエラーメッセージを追加
        ArrayList<String> messages = new ArrayList<String>();
        for (int i = 0; i < errorTypeList.size(); i++) {
            String errorType = errorTypeList.get(i);
            String errorMessage = errorMessageList.get(i);
            for (PdfDemandBean kijiBean : kijiBeanList) {
                String result = kijiBean.getResult();
                if (errorType.equals(result)) {
                    messages.add(errorMessage);
                    break;
                }
            }
        }
        return messages;
    }

    /**
     * PDF要求の成功・失敗件数をカウント
     * (※この件数を元に、JSP側で要求ボタンの表示制御などを行う)
     *
     * @param kijiBeanList 記事リスト
     */
    private void setResultCount(List<PdfDemandBean> kijiBeanList) {
        successCnt = 0;
        errorCnt = 0;
        for (PdfDemandBean kijiBean : kijiBeanList) {
            if (kijiBean.getIsDemandSuccess()) {
                successCnt++;
            } else {
                errorCnt++;
            }
        }
    }

    /**
     * 要求が全件成功しているかを判定する。
     *
     * ※JSP側で使用
     *
     * @return 要求が全件成功している場合にtrue
     */
    public boolean getIsAllSuccess() {
        if (errorCnt == 0) {
            return true;
        }
        return false;
    }

    /**
     * 要求が1件以上成功しているかを判定する。
     *
     * ※JSP側で使用
     *
     * @return 要求が1件以上成功している場合にtrue
     */
    public boolean getSuccessDataExist() {
        if (successCnt > 0) {
            return true;
        }
        return false;
    }

    /**
     * 現在の日時をTimestamp型で返す
     * @return Timestamp型の現在の日時
     * @throws ParseException
     */
    private Timestamp getDate() throws ParseException {
    	return new Timestamp(new SimpleDateFormat(DATE_FORMAT).parse(new SimpleDateFormat(DATE_FORMAT).format(new Date())).getTime());
    }

}
